// FaderPort 2 Component Script
// Copyright (c)2018 PreSonus Software Ltd.

// include SDK files from host
include_file ("resource://com.presonus.musicdevices/sdk/controlsurfacecomponent.js")

// constants missing in current SDK:
Banks.kCustomPrefix = "Custom:";
Banks.kFollow = "FollowBank";

// Encoder mode
const kNavigationMode = 0;
const kLinkMode = 1;
const kPanMode = 2;
const kChannelMode = 3;
const kScrollMode = 4;
const kZoomMode = 5;
const kMasterMode = 6;
const kSectionMode = 7;
const kMarkerMode = 8;
const kLastEncoderMode = kMarkerMode;

// Color definitions
const kLinkColorOff = "white";
const kLinkColorOn = "blue";
const kLinkColorLocked = "red";
const kNormalModeColor = "blue"; // normal mode (pan, scroll)
const kAltModeColor = "white"; // alternative mode (flip, zoom)
const kAutoLatchColor = "red";
const kAutoTrimColor = "red";
const kAutoOffColor = "white";
const kAutoTouchColor = "orange";
const kAutoWriteColor = "red";
const kAutoReadColor = "green";

//************************************************************************************************
// FP2Component
//************************************************************************************************

FP2Component.prototype = new ControlSurfaceComponent ();
function FP2Component ()
{		
	this.onInit = function (hostComponent)
	{
		ControlSurfaceComponent.prototype.onInit.call (this, hostComponent);
		
		// turn on debug output
		//this.debugLog = true;
				
		// keep references to surface elements
		this.model = hostComponent.model;
		let root = this.model.root;
		this.recentParamElement = root.getRecentParamMapping ();
		this.mixerMapping = root.find ("MixerMapping");
		this.channelBankElement = this.mixerMapping.find ("ChannelBankElement");
		
		// channel lock requires Studio One 4.0.1 or later!
		this.channelLockSupported = this.channelBankElement.addToBank != null; 
		this.channelLockBankName = Banks.kCustomPrefix + "FP2LockedChannel";
		//this.log ("Channel lock supported: " + this.channelLockSupported);

		// create parameters
		let paramList = hostComponent.paramList;	

		this.shiftModifer = paramList.addParam ("shiftModifier");
		this.encoderMode = paramList.addInteger (0, kLastEncoderMode, "encoderMode");
		this.lockEncoder = paramList.addParam ("lockEncoder");
		this.flipEncoder = paramList.addParam ("flipEncoder");
		this.scrollZoomMode = paramList.addParam ("scrollZoomMode"); // encoder mode is scroll or zoom
		this.lockChannel = paramList.addParam ("lockChannel");
		this.channelModeOrLock = paramList.addParam ("channelModeOrLock"); // encoder mode is channel or locked channel
		
		this.linkColor = paramList.addColor ("linkColor");
		this.linkColor.fromString (kLinkColorOff);				
		this.scrollZoomColor = paramList.addColor ("scrollZoomColor");
		this.scrollZoomColor.fromString (kNormalModeColor);
		this.panFlipColor = paramList.addColor ("panFlipColor");
		this.panFlipColor.fromString (kNormalModeColor);
		this.channelLinkColor = paramList.addColor ("channelLinkColor");
		this.channelLinkColor.fromString (kLinkColorOn);

		this.linkButtonHandler = paramList.addParam ("linkButtonHandler");
		this.panButtonHandler = paramList.addParam ("panButtonHandler");
		this.channelButtonHandler = paramList.addParam ("channelButtonHandler");
		
		// constant colors
		paramList.addColor ("autoLatchColor").fromString (kAutoLatchColor);
		paramList.addColor ("autoTrimColor").fromString (kAutoTrimColor);
		paramList.addColor ("autoOffColor").fromString (kAutoOffColor);
		paramList.addColor ("autoTouchColor").fromString (kAutoTouchColor);
		paramList.addColor ("autoWriteColor").fromString (kAutoWriteColor);
		paramList.addColor ("autoReadColor").fromString (kAutoReadColor);	
	}
	
	this.isShiftActive = function () { return this.shiftModifer.value != 0; }
	
	this.isEncoderMode = function (mode) { return this.encoderMode.value == mode; }	
	this.setEncoderMode = function (mode) { this.encoderMode.setValue (mode, true); }
	this.toggleEncoderMode = function (onMode, offMode)
	{
		if(this.isEncoderMode (onMode))
			this.setEncoderMode (offMode);
		else
			this.setEncoderMode (onMode);		
	}

	this.isFlipEncoderActive = function () { return this.flipEncoder.value != 0; }	
	this.setFlipEncoder = function (state) { this.flipEncoder.setValue (state, true); }
			
	this.setLockEncoder = function (state) { this.lockEncoder.setValue (state, true); }
	this.setLockChannel = function (state) { this.lockChannel.setValue (state, true); }
			
	this.paramChanged = function (param)
	{
		if(param == this.encoderMode)
		{
			this.updateLinkState ();
			this.updateModeParams ();
		}
		else if(param == this.flipEncoder)
		{
			this.setEncoderMode (kPanMode);
			this.updateModeParams ();		
		}		
		else if(param == this.lockEncoder)
		{						
			this.setEncoderMode (kLinkMode);
			// request host to lock/unlock recent parameter for this device
			this.model.lockRecentParameter (param.value);
		}
		else if(param == this.lockChannel)
		{
			this.setEncoderMode (kChannelMode);			
			this.enterChannelLockMode (this.lockChannel.value);
		}
		// combined handling of user input for convenience functions currently unavailable via XML
		else if(param == this.linkButtonHandler)
		{
			if(this.isEncoderLockActive ()) // turn off secondary function
				this.setLockEncoder (false);
			else
			{
				if(this.isShiftActive ())
					this.setLockEncoder (true);
				else
					this.toggleEncoderMode (kLinkMode, kNavigationMode);
			}
		}
		else if(param == this.panButtonHandler)
		{
			if(this.isFlipEncoderActive ()) // turn off secondary function
				this.setFlipEncoder (false);
			else
			{				
				if(this.isShiftActive ())
					this.setFlipEncoder (true);
				else
					this.toggleEncoderMode (kPanMode, kNavigationMode);
			}
		}
		else if(param == this.channelButtonHandler)
		{
			if(this.isChannelLockActive ()) // turn off secondary function
			{
				if(this.isEncoderMode (kChannelMode) == false) // or go back to channel mode
					this.setEncoderMode (kChannelMode);
				else					
					this.setLockChannel (false);				
			}
			else
			{
				if(this.isShiftActive ())
					this.setLockChannel (true);
				else
					this.toggleEncoderMode (kChannelMode, kNavigationMode);				
			}				
		}
	}
				
	this.enterChannelLockMode = function (state)
	{
		if(this.channelLockSupported)
		{
			if(state)
			{
				// switch from follow bank to locked bank
				if(this.channelBankElement.target == Banks.kFollow && this.channelBankElement.getBankChildCount () > 0)
				{
					//this.log ("Locking current channel");
					let channelStripElement = this.channelBankElement.getElement (0);
					this.channelBankElement.addToBank (this.channelLockBankName, channelStripElement);
					this.channelBankElement.selectBank (this.channelLockBankName);
				}
			}
			else
			{
				// switch from locked bank back to follow bank
				if(this.channelBankElement.target != Banks.kFollow)
				{
					//this.log ("Back to follow mode");
					this.channelBankElement.selectBank (Banks.kFollow);
					this.channelBankElement.emptyBank (this.channelLockBankName);					
				}
			}
			
			this.updateChannelLockState (); // not called automatically for same channel			
		}
		else // not supported by older host versions
			this.lockChannel.value = false;
	}
			
	this.updateChannelLockState = function ()
	{
		if(this.channelLockSupported)
		{
			let locked = false;
			if(this.channelBankElement.target == this.channelLockBankName) // locked bank currently selected
			{
				if(this.channelBankElement.getBankChildCount () > 0)
					locked = true;
				else
				{
					// locked channel has been deleted, must switch back to follow bank
					this.channelBankElement.selectBankDeferred (Banks.kFollow);
				}				
			}
						 
			this.lockChannel.value = locked;
			this.channelLinkColor.fromString (locked ? kLinkColorLocked : kLinkColorOn);

			this.updateModeParams (); // make sure "channelModeOrLock" is updated
		}
	}
	
	this.isChannelLockActive = function ()
	{
		return this.channelLockSupported && this.channelBankElement.target == this.channelLockBankName;
	}
	
	this.updateLockedChannel = function ()
	{
		if(this.isChannelLockActive ())
		{
			this.enterChannelLockMode (false);
			this.enterChannelLockMode (true);				
		}
	}
	
	this.isEncoderLockActive = function ()
	{
		return this.isEncoderMode (kLinkMode) && this.recentParamElement.locked;
	}
	
	this.updateLinkState = function ()
	{
		let mode = this.encoderMode.value;
		if(mode == kLinkMode)
		{
			let locked = this.recentParamElement.locked;
			this.linkColor.fromString (locked ? kLinkColorLocked : kLinkColorOn);			
			HostUtils.setParamMouseOverEnabled (true);
		}
		else
		{
			this.linkColor.fromString (kLinkColorOff);
			HostUtils.setParamMouseOverEnabled (false);	
		}		
	}
	
	this.updateModeParams = function ()
	{
		let mode = this.encoderMode.value;
		let flipped = this.flipEncoder.value;
		this.scrollZoomColor.fromString (mode == kZoomMode ? kAltModeColor : kNormalModeColor);
		this.panFlipColor.fromString (mode == kPanMode && flipped ? kAltModeColor : kNormalModeColor);
		this.scrollZoomMode.value = mode == kZoomMode || mode == kScrollMode;
		this.channelModeOrLock.value = mode == kChannelMode || this.isChannelLockActive ();
	}
}

// factory entry called by host
function createFP2ComponentInstance ()
{
	return new FP2Component;
}
